#ifndef __SANITY_HH__
#define __SANITY_HH__

// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
// all rights reserved.
// licensed to the public under the terms of the GNU GPL (>= 2)
// see the file COPYING for details

#include <stdexcept>

// our assertion / sanity / error logging system was based on GNU Nana,
// but we're only using a small section of it, so we just replicate it
// in place rather than add Yet Another Library Dependency

struct sanity {
  sanity();
  ~sanity();
  void dump_buffer();
  void set_verbose();
  bool verbose;

  char * logbuf;
  size_t pos;
  size_t size;
};

typedef std::runtime_error oops;

extern sanity global_sanity;
void sanity_invariant_handler(char *expr, char *file, int line);

#ifdef __GNUC__
void sanity_printf(sanity & s, const char *format, ...)
  __attribute__((format (printf, 2, 3)));
#else
void sanity_printf(sanity & s, const char *format, ...);
#endif

#define L(f...)                       \
do {                                  \
  sanity_printf (global_sanity, ##f); \
} while(0)

#define I(e)                                                   \
do {                                                           \
  if(!(e)) {                                                   \
    sanity_invariant_handler ("I("#e")", __FILE__, __LINE__);  \
  }                                                            \
} while(0)

#endif // __SANITY_HH__
