
cd $top_builddir
top_builddir=$(pwd)

set -e
TEST=$(basename $0)
PASS=0
FAIL=0
LOG=$top_builddir/$TEST.log
rm -f $LOG

trap 'if [ $FAIL -ne 0 ]; then exit 1; else rm -f $LOG; exit 0; fi' 0

function zfatal(){
    echo "$srcdir/testsuite/$TEST:$1:E:FATAL: " $2  >>$LOG
    FAIL=$(( $FAIL + 1 ))
    exit 1;
}

function zfail(){
    echo "$srcdir/testsuite/$TEST:$1:E:FAIL: " $2  >>$LOG
    FAIL=$(( $FAIL + 1 ))
}

function zpass(){
    echo "$srcdir/testsuite/$TEST:$1:N:PASS: " $2   >>$LOG
    PASS=$(( $PASS + 1 ))
}

function zcheck_exec(){
    set +e
    eval "$2" >>$LOG 2>&1
    STATUS=$?
    if [ $STATUS -eq $3 ]
	then
	zpass $1 "exec status == $STATUS as expected"
	else
	zfail $1 "exec status of '$2', $STATUS != $3"
    fi
    set -e
}

function zcheck_equal(){
    set +e
    if [ "z$2" == "z$3" ]
	then
	zpass $1 "comparison, $2 == $3"
	else
	zfail $1 "comparison, $2 != $3"
    fi
    set -e
}

function zcheck_nonequal(){
    set +e
    if [ "z$2" != "z$3" ]
	then
	zpass $1 "comparison, $2 != $3"
	else
	zfail $1 "comparison, $2 == $3"
    fi
    set -e
}

shopt -s expand_aliases
alias fatal='zfatal $LINENO'
alias fail='zfail $LINENO'
alias pass='zpass $LINENO'
alias check_exec='zcheck_exec $LINENO'
alias check_equal='zcheck_equal $LINENO'
alias check_nonequal='zcheck_nonequal $LINENO'

function need_monotone(){
    if [ ! -x $1/monotone ] ; then fatal "need executable monotone"; fi
}

need_monotone $top_builddir
MT="$top_builddir/monotone --norc --nostd --rcfile=test_hooks.lua --verbose --db=test.db"


function write_test_lua(){
    cat >test_hooks.lua <<EOF
function ignore_file(name)
	if (strfind(name, "%.lua$")) then return 1 end
	if (strfind(name, "%.db$")) then return 1 end
        return nil;
end

function get_passphrase(keyid)
	return keyid;
end

function non_blocking_rng_ok()
	return 1;
end

function persist_phrase_ok()
	return 1;
end

function get_author(branchname)
        local user = getenv("USER");
        local host = getenv("HOSTNAME");
        if ((user == nil) or (host == nil)) then return nil; end
        return format("%s@%s", user, host)
end

function get_newsserver(branchname)
	return "127.0.0.1:8119";
end

function get_newsgroup(branchname)
	return "monotone.test.packets";
end

function get_sender(server, newsgroup)
	return "tester@test.com";
end
EOF
}

function load_test_keys(){
cat >test_keys <<EOF
[pubkey tester@test.net]
MIGdMA0GCSqGSIb3DQEBAQUAA4GLADCBhwKBgQCfN/cAMabgb6T7m8ksGnpQ7LO6hOdnc/7V
yivrRGtmpwSItljht1bmgLQF37KiSPoMEDUb1stfKxaMsYiy8iTyoQ+M2EVFP37n2rtnNZ0H
oVcQd2sRsCerQFh9nslRPymlkQXUlOiNFN6RlFNcdjkucqNe+YorFX21EYw7XuT5XwIBEQ==
[end]
[privkey tester@test.net]
npy0jyqbZdylFkMjdR9OvlqmDHuBGXpGFPt94h96aG+Lp+OdBCmWx8GueHk8FKkexwPqhRBM
PPopUeuwqxuSX+yEodMl5IHBmin0nLnbOeBjtasjemBFEmdNl/jPDF/AeQ2WHhanB731dSQc
vzLOQfKAYmfk56PPULi9oJJNUxaCkvsWtvaI+HUHZrjyyV7dA4Vsc6Jvx2Yf1+MdDAEGk/Rw
ZtP0LmuoiyvRDFqBF8aTmnotyb4kRKohnJ7VF+y6hYvmtMpM3TKnpR7EbojBzNPqKuO7nPUz
jGxA7F84O24Vbf128PNSI5vj4istow26aPjn28qPjfRrkV30WLL/dXfYJkfkTqglYnoEXvF/
xZoVxxNeAX58mgy0A1ErVxv8U7TwuP983GHEpLwy3gbiP+9akAJCr8r823DHmQqq5QDELibP
cuXZfOttpfVRkcbMhjeF0M6czc4HoKgHTAnf/18hzdZwGX/WWvRIBHImbUJ+mDbp2ByDTfKf
ErGXSvZ3HxCqBD8yx1SnXhV8IDHaBmV9wwYcN+H2cxOWGZk7g7xJS19+a3UQB3c3sSXQVJBp
6QpCZgysxkZwzuXDzzLZPT9SLZz4K2p7+7BwMbpy9ZxcyAzmiEtpA24UP06jtjFN7WcXAdx/
E5Gmoe9b1EiXWdReHjUGpc6k0LQ0PPXAwqrcGdwYbOLDZ5xsQ5AsEYSFtyTS60D1nHBcdNmW
M0eOUJFdJf/uNe/2EApc3a8TyEkZtVqiYtOVV3qDB9NmU4bVOkDqzl1F7zJwATWbmasSdkM3
6lxDkczBfCrEjH5p5Y8DU+ge4e4LRtknY9oBOJ7EQO0twYJg3k0=
[end]
EOF
check_exec "$MT read <test_keys" 0
check_exec "rm test_keys" 0
}

function sha1(){
    sha1sum $1 | cut -d ' ' -f 1
}


rm -Rf test
mkdir test
cd test
write_test_lua
load_test_keys
