// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
// all rights reserved.
// licensed to the public under the terms of the GNU GPL (>= 2)
// see the file COPYING for details

#include <boost/regex.hpp>
#include <sstream>

#include "work.hh"
#include "file_io.hh"
#include "sanity.hh"

// working copy / book-keeping file code

string const work_file_name("work");

using namespace boost;

struct add_to_work_set
{    
  work_set & work;
  explicit add_to_work_set(work_set & w) : work(w) {}
  bool operator()(match_results<std::string::const_iterator, regex::alloc_type> const & res) 
  {
    std::string adddel(res[1].first, res[1].second);
    std::string path(res[2].first, res[2].second);
    if (!book_keeping_file(path))
      {
	if (adddel == "-")
	  work.dels.insert(path);
	else if (adddel == "+")
	  work.adds.insert(path);
	else throw oops("unknown add/del character in work set: " + adddel);
      } else {
	throw oops("book-keeping filename: " + path);
      }
    return true;
  }
};

void read_work_set(data const & dat,
		   work_set & work)
{
  regex expr("^[[:space:]]*([\\+\\-])[[:space:]]+([^[:space:]]+)");
  regex_grep(add_to_work_set(work), dat(), expr, match_not_dot_newline);    
}

void write_work_set(data & dat,
		    work_set const & work)
{
  ostringstream tmp;
  for (path_set::const_iterator i = work.dels.begin();
       i != work.dels.end(); ++i)
    tmp << "- " << (*i)() << endl;
  for (path_set::const_iterator i = work.adds.begin();
       i != work.adds.end(); ++i)
    tmp << "+ " << (*i)() << endl;
  dat = tmp.str();
}

void extract_path_set(manifest_map const & man,
		      path_set & paths)
{
  paths.clear();
  for (manifest_map::const_iterator i = man.begin();
       i != man.end(); ++i)
    paths.insert(path_id_pair(*i).path());
}

void apply_work_set(work_set const & work,
		    path_set & paths)
{
  for (path_set::const_iterator i = work.dels.begin();
       i != work.dels.end(); ++i)
    paths.erase(*i);
  for (path_set::const_iterator i = work.adds.begin();
       i != work.adds.end(); ++i)
    paths.insert(*i);
}
